using System;
using System.Web.Compilation;
using System.Resources;
using System.Reflection;
using System.Globalization;
using System.Threading;
using System.Diagnostics;
using System.Runtime.CompilerServices;

namespace Infrastructure.CrossCutting.Resource
{

    /// <summary>
    /// Resource provider for accessing external resources. 
    /// </summary>
    public class GlobalExternalResourceProvider : IResourceProvider
    {
        private string _classKey;
        private string _assemblyName;

        private ResourceManager _resourceManager;

        /// <summary>
        /// Constructs an instance of the provider with the specified
        /// assembly name and resource type. 
        /// </summary>
        /// <param name="classKey">The assembly name and 
        /// resource type in the format [assemblyName]|
        /// [resourceType]</param>
        public GlobalExternalResourceProvider(string classKey)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "GlobalExternalResourceProvider({0})", classKey));

            if (classKey.IndexOf('|') > 0)
            {
                string[] textArray = classKey.Split('|');
                this._assemblyName = textArray[0];
                this._classKey = textArray[1];
            }
            else
                throw new ArgumentException(classKey);


        }

        /// <summary>
        /// Loads the resource assembly and creates a 
        /// ResourceManager instance to access its resources.
        /// If the assembly is already loaded, Load returns a reference
        /// to that assembly.
        /// </summary>
        private void EnsureResourceManager()
        {
            if (this._resourceManager == null)
            {
                lock (this)
                {
                    Assembly asm = Assembly.Load(this._assemblyName);
                    var rm = new ResourceManager(String.Format(CultureInfo.InvariantCulture, "{0}.{1}", this._assemblyName, this._classKey), asm);
                    this._resourceManager = rm;
                }
            }
        }

        #region IResourceProvider Members

        /// <summary>
        /// Retrieves resources using a ResourceManager instance
        /// for the assembly and resource key of this provider 
        /// instance. 
        /// </summary>
        /// <param name="resourceKey">The resource key to find.</param>
        /// <param name="culture">The culture to find.</param>
        /// <returns>The resource value if found.</returns>
        public object GetObject(string resourceKey, System.Globalization.CultureInfo culture)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "GlobalExternalResourceProvider.GetObject({0}, {1})", resourceKey, culture));

            this.EnsureResourceManager();
            if (culture == null)
            {
                culture = CultureInfo.CurrentUICulture;
            }
            return this._resourceManager.GetObject(resourceKey, culture);

        }

        /// <summary>
        /// Implicit expressions are not supported by this provider 
        /// therefore a ResourceReader need not be implemented.
        /// Throws a NotSupportedException.
        /// </summary>
        public System.Resources.IResourceReader ResourceReader
        {
            get
            {
                Debug.WriteLine("GlobalExternalResourceProvider.get_ResourceReader()");

                throw new NotSupportedException();
            }

        }

        #endregion

    }

}