﻿using System.Linq.Expressions;
using Domain.Core;
using Domain.Core.Specification;

namespace Infrastructure.Data.Core
{
    using System;
    using System.Collections.Generic;
    using System.Data.Entity;
    using System.Linq;

    /// <summary>
    /// Repository base class
    /// </summary>
    /// <typeparam name="TEntity">The type of underlying entity in this repository</typeparam>
    /// <typeparam name="TEntityKey"> the key of Entity</typeparam>
    public abstract class Repository<TEntity,TEntityKey> : IRepository<TEntity, TEntityKey> where TEntity:class 
    {
        #region Members
        readonly IQueryableUnitOfWork _unitOfWork;
        #endregion
        #region private
        private DbSet<TEntity> GetSet()
        {
            return _unitOfWork.CreateSet<TEntity>();
        }
        #endregion
        #region Constructor

        /// <summary>
        /// Create a new instance of repository
        /// </summary>
        /// <param name="unitOfWork">Associated Unit Of Work</param>
        protected Repository(IQueryableUnitOfWork unitOfWork)
        {
            if (unitOfWork == (IUnitOfWork)null)
                throw new ArgumentNullException("unitOfWork");

            _unitOfWork = unitOfWork;
        }

        #endregion

        #region IRepository Members
        public IUnitOfWork UnitOfWork
        {
            get 
            {
                return _unitOfWork;
            }
        }
        
        public virtual void Add(TEntity item)
        {

            if (item != (TEntity)null)
                GetSet().Add(item); // add new item in this set
            else
            {
                //add the log
            }
            
        }
        
        public virtual void Remove(TEntity item)
        {
            if (item != (TEntity)null)
            {
                //attach item if not exist
                _unitOfWork.Attach(item);
                //set as "removed"
                GetSet().Remove(item);
            }
            else
            {
                //add the log
            }
        }
        
        public virtual void TrackItem(TEntity item)
        {
            if (item != (TEntity)null)
                _unitOfWork.Attach<TEntity>(item);
            else
            {
                //add the log
            }
        }
        
        public virtual void Modify(TEntity item)
        {
            if (item != (TEntity)null)
                _unitOfWork.SetModified(item);
            else
            {
                //add the log
            }
        }

        public virtual IEnumerable<TEntity> GetAll()
        {
            return GetSet();
        }
        public virtual IEnumerable<TEntity> AllMatching(ISpecification<TEntity> specification)
        {
            return GetSet().Where(specification.SatisfiedBy());
        }
        
        public virtual IEnumerable<TEntity> GetFiltered(Expression<Func<TEntity, bool>> filter)
        {
            return GetSet().Where(filter);
        }

        public virtual void Merge(TEntity persisted, TEntity current)
        {
            _unitOfWork.ApplyCurrentValues(persisted, current);
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// <see cref="M:System.IDisposable.Dispose"/>
        /// </summary>
        public void Dispose()
        {
            if (_unitOfWork != null)
                _unitOfWork.Dispose();
        }

        #endregion

        #region abstract
        public abstract TEntity Get(TEntityKey id);
        #endregion

    }
}
