using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web.Mvc;
using Infrastructure.CrossCutting.IoC;

namespace Presentation.Web.UI.BootStrapper
{
    /// <summary>
    /// Default boot strapper implementation
    /// <see cref="NLayerApp.Presentation.Web.MVC.Client.Extensions.BootStrapper.IBootStrapper"/>
    /// </summary>
    public class DefaultBootStrapper: IBootStrapper
    {
        #region Members

        /// <summary>
        /// Current IoC container
        /// </summary>
        readonly IContainer _currentContainer;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new instance of this boot strapper
        /// </summary>
        /// <param name="container">Default IoC container</param>
        public DefaultBootStrapper(IContainer container)
        {
            if (container == (IContainer)null)
                throw new ArgumentNullException("container");

            _currentContainer = container;
        }

        #endregion

        #region IBootStrapper Members

        /// <summary>
        /// <see cref="M:NLayerApp.Presentation.Web.MVC.Client.Extensions.BootStrapper.IBootStrapper.Boot"/>
        /// </summary>
        public void Boot()
        {
            //Register controllers into MVC infrastructure
            RegisterControllers();

            //Register new model binders
            RegisterModelBinders();

            //register factories
            RegisterFactories();
        }

        #endregion

        #region Private Methods

        private void RegisterFactories()
        {
            //Create a custom controller factory to resolve controllers with IoC container
            IControllerFactory factory = new IoCControllerFactory(_currentContainer);

            //Set new controller factory in ASP.MVC Controller builder
            ControllerBuilder.Current.SetControllerFactory(factory);
        }

        private void RegisterModelBinders()
        {
            //Register a new model binder for customers. This model binder enables the deserialization
            //of a given customer in edit scenarios.
            //ModelBinders.Binders.Add(typeof(Customer), new SelfTrackingEntityModelBinder<Customer>());


            //Register a new model binder for customer's picture. This model binder binds the posted
            //image to a the byte array field in the CustomerPicture class.
        }

        private void RegisterControllers()
        { 
            //In this implementations only controllers in same assembly are registered in IoC container. If you
            // have controllers in a different assembly check this code.

            //Recover excuting assembly.
            Assembly assembly =Assembly.GetExecutingAssembly();

            //Recover all controller types in this assembly.
            IEnumerable<Type> controllers = assembly.GetExportedTypes()
                                                    .Where(x => typeof(IController).IsAssignableFrom(x));
            //Register all controllers types
            foreach (Type item in controllers)
                _currentContainer.RegisterType(item);
        }

        #endregion 
    }
}
